@@ -111,7 +111,7 @@ int handle__connack(struct mosquitto *mosq)
 		case 0:
 			pthread_mutex_lock(&mosq->state_mutex);
 			if(mosq->state != mosq_cs_disconnecting){
-				mosq->state = mosq_cs_connected;
+				mosq->state = mosq_cs_active;
 			}
 			pthread_mutex_unlock(&mosq->state_mutex);
 			message__retry_check(mosq);
