@@ -37,9 +37,12 @@ Contributors:
 
 int handle__pingreq(struct mosquitto *mosq)
 {
+	int state;
+
 	assert(mosq);
 
-	if(mosq->state != mosq_cs_connected){
+	state = mosquitto__get_state(mosq);
+	if(state != mosq_cs_active){
 		return MOSQ_ERR_PROTOCOL;
 	}
 
@@ -57,11 +60,8 @@ int handle__pingresp(struct mosquitto *mosq)
 
 	assert(mosq);
 
-	pthread_mutex_lock(&mosq->state_mutex);
-	state = mosq->state;
-	pthread_mutex_unlock(&mosq->state_mutex);
-
-	if(state != mosq_cs_connected){
+	state = mosquitto__get_state(mosq);
+	if(state != mosq_cs_active){
 		return MOSQ_ERR_PROTOCOL;
 	}
 
