@@ -29,14 +29,15 @@ Contributors:
 #include "mosquitto.h"
 #include "mosquitto_internal.h"
 #include "logging_mosq.h"
-#include "mqtt3_protocol.h"
+#include "mqtt_protocol.h"
 #include "memory_mosq.h"
 #include "net_mosq.h"
 #include "packet_mosq.h"
+#include "property_mosq.h"
 #include "send_mosq.h"
 
 
-int send__publish(struct mosquitto *mosq, uint16_t mid, const char *topic, uint32_t payloadlen, const void *payload, int qos, bool retain, bool dup)
+int send__publish(struct mosquitto *mosq, uint16_t mid, const char *topic, uint32_t payloadlen, const void *payload, int qos, bool retain, bool dup, const mosquitto_property *cmsg_props, const mosquitto_property *store_props, uint32_t expiry_interval)
 {
 #ifdef WITH_BROKER
 	size_t len;
@@ -50,7 +51,6 @@ int send__publish(struct mosquitto *mosq, uint16_t mid, const char *topic, uint3
 #endif
 #endif
 	assert(mosq);
-	assert(topic);
 
 #if defined(WITH_BROKER) && defined(WITH_WEBSOCKETS)
 	if(mosq->sock == INVALID_SOCKET && !mosq->wsi) return MOSQ_ERR_NO_CONN;
@@ -110,7 +110,7 @@ int send__publish(struct mosquitto *mosq, uint16_t mid, const char *topic, uint3
 					}
 					log__printf(NULL, MOSQ_LOG_DEBUG, "Sending PUBLISH to %s (d%d, q%d, r%d, m%d, '%s', ... (%ld bytes))", mosq->id, dup, qos, retain, mid, mapped_topic, (long)payloadlen);
 					G_PUB_BYTES_SENT_INC(payloadlen);
-					rc =  send__real_publish(mosq, mid, mapped_topic, payloadlen, payload, qos, retain, dup);
+					rc =  send__real_publish(mosq, mid, mapped_topic, payloadlen, payload, qos, retain, dup, cmsg_props, store_props, expiry_interval);
 					mosquitto__free(mapped_topic);
 					return rc;
 				}
@@ -124,26 +124,63 @@ int send__publish(struct mosquitto *mosq, uint16_t mid, const char *topic, uint3
 	log__printf(mosq, MOSQ_LOG_DEBUG, "Client %s sending PUBLISH (d%d, q%d, r%d, m%d, '%s', ... (%ld bytes))", mosq->id, dup, qos, retain, mid, topic, (long)payloadlen);
 #endif
 
-	return send__real_publish(mosq, mid, topic, payloadlen, payload, qos, retain, dup);
+	return send__real_publish(mosq, mid, topic, payloadlen, payload, qos, retain, dup, cmsg_props, store_props, expiry_interval);
 }
 
 
-int send__real_publish(struct mosquitto *mosq, uint16_t mid, const char *topic, uint32_t payloadlen, const void *payload, int qos, bool retain, bool dup)
+int send__real_publish(struct mosquitto *mosq, uint16_t mid, const char *topic, uint32_t payloadlen, const void *payload, int qos, bool retain, bool dup, const mosquitto_property *cmsg_props, const mosquitto_property *store_props, uint32_t expiry_interval)
 {
 	struct mosquitto__packet *packet = NULL;
 	int packetlen;
+	int proplen = 0, varbytes;
 	int rc;
+	mosquitto_property expiry_prop;
 
 	assert(mosq);
-	assert(topic);
 
-	packetlen = 2+strlen(topic) + payloadlen;
+	if(topic){
+		packetlen = 2+strlen(topic) + payloadlen;
+	}else{
+		packetlen = 2 + payloadlen;
+	}
 	if(qos > 0) packetlen += 2; /* For message id */
+	if(mosq->protocol == mosq_p_mqtt5){
+		proplen = 0;
+		proplen += property__get_length_all(cmsg_props);
+		proplen += property__get_length_all(store_props);
+		if(expiry_interval > 0){
+			expiry_prop.next = NULL;
+			expiry_prop.value.i32 = expiry_interval;
+			expiry_prop.identifier = MQTT_PROP_MESSAGE_EXPIRY_INTERVAL;
+			expiry_prop.client_generated = false;
+
+			proplen += property__get_length_all(&expiry_prop);
+		}
+
+		varbytes = packet__varint_bytes(proplen);
+		if(varbytes > 4){
+			/* FIXME - Properties too big, don't publish any - should remove some first really */
+			cmsg_props = NULL;
+			store_props = NULL;
+			expiry_interval = 0;
+		}else{
+			packetlen += proplen + varbytes;
+		}
+	}
+	if(packet__check_oversize(mosq, packetlen)){
+#ifdef WITH_BROKER
+		log__printf(NULL, MOSQ_LOG_NOTICE, "Dropping too large outgoing PUBLISH for %s (%d bytes)", mosq->id, packetlen);
+#else
+		log__printf(NULL, MOSQ_LOG_NOTICE, "Dropping too large outgoing PUBLISH (%d bytes)", packetlen);
+#endif
+		return MOSQ_ERR_OVERSIZE_PACKET;
+	}
+
 	packet = mosquitto__calloc(1, sizeof(struct mosquitto__packet));
 	if(!packet) return MOSQ_ERR_NOMEM;
 
 	packet->mid = mid;
-	packet->command = PUBLISH | ((dup&0x1)<<3) | (qos<<1) | retain;
+	packet->command = CMD_PUBLISH | ((dup&0x1)<<3) | (qos<<1) | retain;
 	packet->remaining_length = packetlen;
 	rc = packet__alloc(packet);
 	if(rc){
@@ -151,11 +188,24 @@ int send__real_publish(struct mosquitto *mosq, uint16_t mid, const char *topic,
 		return rc;
 	}
 	/* Variable header (topic string) */
-	packet__write_string(packet, topic, strlen(topic));
+	if(topic){
+		packet__write_string(packet, topic, strlen(topic));
+	}else{
+		packet__write_uint16(packet, 0);
+	}
 	if(qos > 0){
 		packet__write_uint16(packet, mid);
 	}
 
+	if(mosq->protocol == mosq_p_mqtt5){
+		packet__write_varint(packet, proplen);
+		property__write_all(packet, cmsg_props, false);
+		property__write_all(packet, store_props, false);
+		if(expiry_interval > 0){
+			property__write_all(packet, &expiry_prop, false);
+		}
+	}
+
 	/* Payload */
 	if(payloadlen){
 		packet__write_bytes(packet, payload, payloadlen);
