@@ -22,6 +22,7 @@ Contributors:
 
 #include "mosquitto_internal.h"
 #include "net_mosq.h"
+#include "util_mosq.h"
 
 void *mosquitto__thread_main(void *obj);
 
@@ -89,9 +90,7 @@ void *mosquitto__thread_main(void *obj)
 	if(!mosq) return NULL;
 
 	do{
-		pthread_mutex_lock(&mosq->state_mutex);
-		state = mosq->state;
-		pthread_mutex_unlock(&mosq->state_mutex);
+		state = mosquitto__get_state(mosq);
 		if(state == mosq_cs_new){
 #ifdef WIN32
 			Sleep(10);
@@ -103,10 +102,6 @@ void *mosquitto__thread_main(void *obj)
 		}
 	}while(1);
 
-	if(state == mosq_cs_connect_async){
-		mosquitto_reconnect(mosq);
-	}
-
 	if(!mosq->keepalive){
 		/* Sleep for a day if keepalive disabled. */
 		mosquitto_loop_forever(mosq, 1000*86400, 1);
