@@ -33,6 +33,9 @@ int mosquitto__check_keepalive(struct mosquitto *mosq);
 uint16_t mosquitto__mid_generate(struct mosquitto *mosq);
 FILE *mosquitto__fopen(const char *path, const char *mode, bool restrict_read);
 
+int mosquitto__set_state(struct mosquitto *mosq, enum mosquitto_client_state state);
+enum mosquitto_client_state mosquitto__get_state(struct mosquitto *mosq);
+
 #ifdef WITH_TLS
 int mosquitto__hex2bin_sha1(const char *hex, unsigned char **bin);
 int mosquitto__hex2bin(const char *hex, unsigned char *bin, int bin_max_len);
