@@ -271,6 +271,7 @@ void config__init(struct mosquitto_db *db, struct mosquitto__config *config)
 	config->default_listener.max_connections = -1;
 	config->default_listener.protocol = mp_mqtt;
 	config->default_listener.security_options.allow_anonymous = -1;
+	config->default_listener.security_options.allow_zero_length_clientid = true;
 	config->default_listener.maximum_qos = 2;
 	config->default_listener.max_topic_alias = 10;
 }
@@ -471,6 +472,7 @@ int config__parse_args(struct mosquitto_db *db, struct mosquitto__config *config
 			|| config->default_listener.security_options.psk_file
 			|| config->default_listener.security_options.auth_plugin_config_count
 			|| config->default_listener.security_options.allow_anonymous != -1
+			|| config->default_listener.security_options.allow_zero_length_clientid != true
 			){
 
 		config->listener_count++;
@@ -529,6 +531,7 @@ int config__parse_args(struct mosquitto_db *db, struct mosquitto__config *config
 		config->listeners[config->listener_count-1].security_options.auth_plugin_configs = config->default_listener.security_options.auth_plugin_configs;
 		config->listeners[config->listener_count-1].security_options.auth_plugin_config_count = config->default_listener.security_options.auth_plugin_config_count;
 		config->listeners[config->listener_count-1].security_options.allow_anonymous = config->default_listener.security_options.allow_anonymous;
+		config->listeners[config->listener_count-1].security_options.allow_zero_length_clientid = config->default_listener.security_options.allow_zero_length_clientid;
 	}
 
 	/* Default to drop to mosquitto user if we are privileged and no user specified. */
@@ -638,6 +641,7 @@ int config__read(struct mosquitto_db *db, struct mosquitto__config *config, bool
 		config__init_reload(db, &config_reload);
 		config_reload.listeners = config->listeners;
 		config_reload.listener_count = config->listener_count;
+		cur_security_options = NULL;
 		rc = config__read_file(&config_reload, reload, db->config_file, &cr, 0, &lineno);
 	}else{
 		rc = config__read_file(config, reload, db->config_file, &cr, 0, &lineno);
@@ -1388,6 +1392,7 @@ int config__read_file_core(struct mosquitto__config *config, bool reload, struct
 						}
 
 						cur_listener->security_options.allow_anonymous = -1;
+						cur_listener->security_options.allow_zero_length_clientid = true;
 						cur_listener->protocol = mp_mqtt;
 						cur_listener->port = tmp_int;
 						cur_listener->maximum_qos = 2;
